"""Top-level package for lotecc."""
from lotecc.lotecc import lote_chinese_conversion

__all__ = ['lote_chinese_conversion']

__author__ = """Allen Shaw"""
__email__ = 'lonsty@sina.com'
__version__ = '0.1.1'



"""Console script for lotecc."""
import sys
import click

from lotecc.lotecc import lote_chinese_conversion


@click.command()
@click.option('-c', '--conversion', 'conversion', default='s2t', show_default=True,
              help='Conversion method between Simplified Chinese and Traditional Chinese.')
@click.option('-i', '--input', 'input', default='.',
              help='Input file or directory. The default is current dir.')
@click.option('-o', '--output', 'output', default=None,
              help='Output file or directory. The default is the same as the source file, '
                   'which means that the source file will be overwritten.')
@click.option('-q', '--quiet', 'quiet', is_flag=True, show_default=True,
              help='Disable screen output.')
@click.option('--ignore', 'ignore', default='.gitignore', show_default=True,
              help='Can be a .gitignore syntax file. Also can be one or more patterns, separated by commas.')
@click.option('--suffix', 'suffix', default=None, show_default=False,
              help='Suffix of output filename, add this to keep both the source file and converted file.')
@click.option('--in-enc', 'in_enc', default='utf-8', show_default=True, help='Encoding for input.')
@click.option('--out-enc', 'out_enc', default='utf-8', show_default=True, help='Encoding for output.')
def main(**kwargs):
    """Convert files between Simplified Chinese and Traditional Chinese.

    Supported conversions:

        hk2s: Traditional Chinese (Hong Kong standard) to Simplified Chinese\n
        s2hk: Simplified Chinese to Traditional Chinese (Hong Kong standard)\n
        s2t: Simplified Chinese to Traditional Chinese\n
        s2tw: Simplified Chinese to Traditional Chinese (Taiwan standard)\n
        s2twp: Simplified Chinese to Traditional Chinese (Taiwan standard, with phrases)\n
        t2hk: Traditional Chinese to Traditional Chinese (Hong Kong standard)\n
        t2s: Traditional Chinese to Simplified Chinese\n
        t2tw: Traditional Chinese to Traditional Chinese (Taiwan standard)\n
        tw2s: Traditional Chinese (Taiwan standard) to Simplified Chinese\n
        tw2sp: Traditional Chinese (Taiwan standard) to Simplified Chinese (with phrases)\n
    """

    try:
        converted = lote_chinese_conversion(**kwargs)
    except Exception as e:
        click.echo(e)
        return 1

    if not kwargs.get('quiet', False):
        for input_file, output_file in converted:
            print(input_file + '\t-->\t' + output_file)

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python

"""Tests for `lotecc` package."""
import os

import pytest

from click.testing import CliRunner

from lotecc import lotecc
from lotecc import cli


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_command_line_interface():
    """Test the CLI."""

    runner = CliRunner()
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert 'Show this message and exit.' in help_result.output


def test_command_line_s2t():
    """Test s2t."""

    input_file = 'tests/testcases/s2t.in'
    output_file = 'tests/testcases/s2t.out'
    input_ans = 'tests/testcases/s2t.ans'

    runner = CliRunner()
    conversion_result = runner.invoke(cli.main, ['-c', 's2t', '-i', input_file, '-o', output_file])
    assert conversion_result.exit_code == 0

    assert os.path.isfile(output_file)

    with open(output_file, 'r') as f:
        output_str = f.read().strip()
    with open(input_ans, 'r') as f:
        answer_str = f.read().strip()

    assert output_str == answer_str

    try:
        os.remove(output_file)
    except FileNotFoundError:
        pass


def test_command_line_t2s():
    """Test t2s."""

    input_file = 'tests/testcases/t2s.in'
    output_file = 'tests/testcases/t2s.out'
    input_ans = 'tests/testcases/t2s.ans'

    runner = CliRunner()
    conversion_result = runner.invoke(cli.main, ['-c', 't2s', '-i', input_file, '-o', output_file])
    assert conversion_result.exit_code == 0

    assert os.path.isfile(output_file)

    with open(output_file, 'r') as f:
        output_str = f.read().strip()
    with open(input_ans, 'r') as f:
        answer_str = f.read().strip()

    assert output_str == answer_str

    try:
        os.remove(output_file)
    except FileNotFoundError:
        pass


def test_command_line_lote_s2t():
    """Test lote s2t"""

    input_dir = 'tests/testcases'
    output_files = [
        'tests/testcases/s2t_c.ans',
        'tests/testcases/s2t_c.in',
        'tests/testcases/t2s_c.ans',
        # 'tests/testcases/t2s_c.in',
        ''
    ]
    answer_files = [
        # 'tests/testcases/s2t.ans',
        '',
        'tests/testcases/s2t.ans',
        'tests/testcases/t2s.in',
        # 'tests/testcases/t2s.in',
        '',
    ]

    runner = CliRunner()
    conversion_result = runner.invoke(cli.main, ['-i', input_dir, '--suffix', '_c'])
    assert conversion_result.exit_code == 0

    for idx, output_file in enumerate(output_files):
        answer_file = answer_files[idx]
        if output_file and answer_file:
            assert os.path.isfile(output_file)

            with open(output_file, 'r') as f:
                output_str = f.read().strip()
            with open(answer_file, 'r') as f:
                answer_str = f.read().strip()

            assert output_str == answer_str

        try:
            os.remove(output_file)
        except FileNotFoundError:
            pass


def test_lotecc_s2t():
    ret = lotecc.lote_chinese_conversion(conversion='s2t',
                                         input='tests/testcases/',
                                         output=None,
                                         in_enc='utf-8',
                                         out_enc='utf-8',
                                         suffix='_c',
                                         ignore='.gitignore')
    assert len(ret) == 3

    for _, output_file in ret:
        try:
            os.remove(output_file)
        except FileNotFoundError:
            pass


def test_lotecc_bin():
    input_file = 'tests/testcases/404.png'

    runner = CliRunner()
    conversion_result = runner.invoke(cli.main, ['-i', input_file])

    assert conversion_result.exit_code == 0
    assert conversion_result.output == ''



"""Main module."""
import os
import re
from fnmatch import fnmatch

from opencc import OpenCC
from pydantic import BaseModel, validator


def check_file_exist(filename: str) -> str:
    """
    Check if the file exists in the current directory or the upper directories.

    :param filename: type str, the filename.
    :return: type str, return abspath of the file if exist, else a empty string.
    """
    if os.path.isfile(filename):
        return os.path.abspath(filename)
    else:
        dirs = os.path.split(os.getcwd())
        for i in range(len(dirs), 0, -1):
            full_path = os.path.join(*dirs[:i], filename)
            if os.path.isfile(full_path):
                return full_path
    return ''


def read_ignores(ignore_file: str) -> list:
    """
    Read ignore patterns from a .gitignore syntax file.

    :param ignore_file: type str, just the name of the ignore file,
                        may not be the correct path.
    :return: type list, a list of patterns.
    """

    ignore_file_path = check_file_exist(ignore_file)
    if not ignore_file_path:
        return []

    with open(ignore_file_path, 'r') as f:
        lines = f.readlines()

    pt = re.compile(r'\S')
    ignores = [l.strip().replace('/', '') for l in lines
               if pt.match(l) and not l.strip().startswith('#')]
    ignores.append('.git')

    return ignores


def get_list_of_files(dir_name: str, ignores: list) -> list:
    """
    Get all files in a directory excluding ignored files.

    :param dir_name: type str, the root directory.
    :param ignores: type str, the patterns to exclude.
    :return: type list, a list of files excluding ignored files.
    """
    list_of_file = os.listdir(dir_name)
    all_files = []

    for entry in list_of_file:
        full_path = os.path.abspath(os.path.join(dir_name, entry))
        for pattern in ignores:
            if fnmatch(os.path.split(full_path)[-1], pattern):
                break
        else:
            if os.path.isdir(full_path):
                all_files = all_files + get_list_of_files(full_path, ignores)
            else:
                all_files.append(full_path)

    return all_files


class LoteccConfig(BaseModel):
    conversion: str = 's2t'
    input: str = '.'
    output: str = None
    in_enc: str = 'utf-8'
    out_enc: str = 'utf-8'
    suffix: str = None
    ignore: str = '.gitignore'

    @property
    def ignore_patterns(self) -> list:
        """
        Read ignore patterns from file, or get from input string.

        :return: type list, .gitignore syntax patterns.
        """
        return read_ignores(self.ignore) or self.ignore.split(',')

    @property
    def input_files(self) -> list:
        """
        List files filtered by ignore patterns in input directory, or a input file.

        :return: type list, files to convert.
        """
        if os.path.isdir(self.input):
            return get_list_of_files(self.input, self.ignore_patterns)
        elif os.path.isfile(self.input):
            return [os.path.abspath(self.input)]
        else:
            raise ValueError(f'<{self.input}> is not a file or directory')

    @validator('conversion')
    def valid_conversion(cls, v: str) -> str:
        """
        Validate argument of conversion.

        :param v: type str, the value of conversion.
        :return: type str, the valid value of conversion.
        :raise: raise ValueError when conversion is not supported.
        """
        if v.lower() in ['hk2s', 's2hk', 's2t', 's2tw', 's2twp',
                         't2hk', 't2s', 't2tw', 'tw2s', 'tw2sp']:
            return v.lower()
        else:
            raise ValueError(f'Error: conversion <{v}> not support')


def lote_chinese_conversion(**kwargs):
    """
    Convert files between Simplified Chinese and Traditional Chinese.

    :param conversion: type str, default 's2t', the conversion method.
    :param input: type str, default '.', an input file or a directory.
    :param output: type str, default None, an output file or a directory.
    :param in_enc: type str, default 'utf-8', encoding for input.
    :param out_enc: type str, default 'utf-8', encoding for output.
    :param suffix: type str, default None, suffix of output filename.
    :param ignore: type str, default '.gitignore', a .gitignore syntax file,
                   or patterns, separated by commas.
    :return: type list, a list of tuples, tuple contains source file and converted file.
    """
    config = LoteccConfig(**kwargs)
    cc = OpenCC(config.conversion)
    converted = []

    for input_file in config.input_files:
        try:
            with open(input_file, encoding=config.in_enc) as f:
                input_str = f.read()
        except UnicodeDecodeError:
            continue

        output_str = cc.convert(input_str)

        output_file = input_file

        if config.output:
            if os.path.isdir(config.output):
                output_file = os.path.abspath(os.path.join(config.output,
                                                           os.path.basename(input_file)))
            else:
                output_file = os.path.abspath(config.output)

        if config.suffix:
            name, extension = os.path.splitext(output_file)
            output_file = name + config.suffix + extension

        if output_str == input_str:
            continue

        with open(output_file, 'w', encoding=config.out_enc) as f:
            f.write(output_str)
            converted.append((input_file, output_file))

    return converted
